package com.github.jchanghong

import cn.hutool.core.io.FileTypeUtil
import cn.hutool.core.io.FileUtil
import cn.hutool.core.util.XmlUtil
import cn.hutool.http.HttpUtil
import com.github.jchanghong.hikcomponent.HikComponent
import org.gradle.api.Project
import org.gradle.api.tasks.Sync
import org.gradle.api.tasks.diagnostics.DependencyReportTask
import org.springframework.boot.gradle.tasks.bundling.BootJar
import org.w3c.dom.Element
import java.io.File
import java.util.*

internal fun addMyTasks(project: Project, myExtension: JchPluginExtension) {
    for (softwareComponent in project.components) {
        log2(softwareComponent.name, project, myExtension.logInfo)
    }
    // Register a task
    project.tasks.register("readPom") { task ->
        task.group = "jch"
        task.description = "pom依赖读取，加入build文件"
        task.doLast {
            val pomFile = File(project.projectDir, "pom.xml")
            val gradleFile = File(project.projectDir, "build.gradle.kts")
            if (!pomFile.exists()) return@doLast
            if (!gradleFile.exists()) return@doLast
            val readText = pomFile.readText()
            if (readText.isNullOrBlank()) return@doLast
            val parseXml = XmlUtil.parseXml(readText).childNodes.item(0)
            val element = XmlUtil.getElement(parseXml as Element?, "dependencies")
            val depenList = XmlUtil.getElements(element, "dependency")
//     <groupId>org.springframework.cloud</groupId>
//            <artifactId>spring-cloud-starter-consul-discovery</artifactId>
//            <version>2.1.3.RELEASE</version>
            val joinToString = depenList.map {
                val groupId = XmlUtil.getElements(it, "groupId")?.firstOrNull()?.textContent
                val artifactId = XmlUtil.getElements(it, "artifactId")?.firstOrNull()?.textContent
                val version = XmlUtil.getElements(it, "version")?.firstOrNull()?.textContent
                val s = if (version.isNullOrBlank()) "" else ":$version"
                """implementation("${groupId}:${artifactId}$s")"""
            }.joinToString(separator = "\n")
            val dependencies = """
                dependencies {
                   $joinToString
                }
            """.trimIndent()
            val readText1 = gradleFile.readText()
            gradleFile.writeText(readText1 + "\n" + dependencies)
        }
    }

    project.tasks.register("showdep", DependencyReportTask::class.java) {
        it.group = "jch"
        it.description = "只显示编译器依赖"
        it.setConfiguration("compileClasspath")
    }
    project.tasks.register("showLatestVersion") {
        it.group = "jch"
        it.description = "获取maven依赖的最新版本，aliyun服务器上"
        it.doLast {
            val mavenLocal = it.project.repositories.mavenLocal()
            val mavenroot = mavenLocal.url.path.removePrefix("/")
            val configurationsSet = setOf("api", "implementation", "compile", "testApi", "testImplementation")
            val allDepens = project.configurations.filter { it.name in configurationsSet }.flatMap { conf ->
                conf.dependencies.map { "${it.group}:${it.name}" }
            }.toHashSet()
            for (depen in allDepens) {
                project.logger.quiet(depen + getMavenVersionAndTime(depen, mavenroot, project))
            }
        }
    }
    project.tasks.register("copyHikvision", Sync::class.java) {
        val destDir = File(project.buildDir, "mavenHikvision").absolutePath
        it.group = "jch"
        it.description = "复制mavenlocal中hik依赖到build目录"
        val mavenLocal = it.project.repositories.mavenLocal()
        val mavenroot = mavenLocal.url.path.removePrefix("/")
        it.doFirst {
            FileHelper.removeMavenLastAndRemoteFiles(mavenroot, project)
            project.logger.quiet(mavenroot)
            project.logger.quiet(destDir)
        }
        it.from(mavenroot) {
            it.includeEmptyDirs = false
            it.eachFile {
                if (!it.path.isHikvisionPath(myExtension.hikvsionMavenNameLikeset)) {
                    it.exclude()
                } else {
                    project.logger.quiet(it.path)
                }
            }
        }
        it.into(destDir)
    }

    project.tasks.register("fenkai") {
        it.group = "jch"
        it.description = "分开java和kotlin文件"
        it.doLast {
            val main = project.file("src/main")
            val javapath = """src\main\java"""
            val kotlipath = """src\main\kotlin"""
            main.walkBottomUp().filter {
                it.name.endsWith(".java", false)
                        || it.name.endsWith(".kt", false)
            }.forEach {
                if (it.absolutePath.endsWith(".kt", false)
                        && it.absolutePath.contains(javapath, false)
                ) {
                    FileUtil.move(it, File(it.absolutePath.replace(javapath, kotlipath)), true)
                    project.logger.quiet(it.absolutePath)
                }
                if (it.absolutePath.endsWith(".java", false)
                        && it.absolutePath.contains(kotlipath, false)
                ) {
                    FileUtil.move(it, File(it.absolutePath.replace(kotlipath, javapath)), true)
                    project.logger.quiet(it.absolutePath)
                }
            }
        }
    }

    project.tasks.register("windowsFileFenLei") {
        it.group = "jch"
        it.description = "windows文件自动分类"
        it.doLast {
            windows10fileAutoFenlei(project)
        }
    }
    project.tasks.register("winsw") {
        it.group = "jch"
        it.description = "产生 生成 windows服务 需要的文件"
        it.doLast {
            winsw(project)
        }
    }
    project.tasks.register("codeGenerator") {
        it.group = "jch"
        it.description = "代码生成，输入表名，生成常用代码，mybatisplus"
        it.doLast {
            CodeGenerator.main2(emptyArray(), project)
        }
    }

    project.tasks.register("hikComponent") {
        it.dependsOn("bootJar")
        it.group = "jch"
        it.description = "生成hikComponent"
        val firstOrNull = project.tasks.withType(BootJar::class.java).firstOrNull()
        if (firstOrNull != null) {
            it.inputs.file(firstOrNull.outputs.files.first().absolutePath)
        }
        val outDir = File(project.buildDir, "componentname_1.0.100.20200408205737").absolutePath
        it.outputs.dir(outDir)
        it.doLast {
            HikComponent.createHikComponent(project, outDir,myExtension)
        }
    }
    project.tasks.register("windowsFileToLinux") {
        it.group = "jch"
        it.description = """windows换行符换成linux"""
        it.doLast {
            windowsFileToLinux(project,myExtension)
        }
    }
}

fun windowsFileToLinux(project: Project, myExtension: JchPluginExtension) {
    val path = project.properties.get("jch.windowsFileToLinuxPath")?.toString() ?: ""
    if (path.isNullOrBlank()||!FileUtil.isDirectory(path)) {
        log2("jch.windowsFileToLinuxPath not exit or not path",project,true)
        return
    }
    File(path).walkTopDown().filter { it.isFile }
            .forEach {
                windowsLineToLinux(it, project, myExtension)
            }
}

fun windowsLineToLinux(it: File, project: Project, myExtension: JchPluginExtension) {
    if ((it.length() / 1024) > 10000) return
    if (FileTypeUtil.getType(it) in setOf("jar", "zip")) return
//                log2(it.absolutePath+"     "+FileTypeUtil.getType(it)+ "  "+ it.length()/1024,project,true)
    val readText = it.readText()
    if ("\r\n" in readText) {
        it.writeText(readText.replace("\r\n", "\n"))
        log2(it.absolutePath+"  windowsLineToLinux", project, true)
    }
}

/** 根据group:name 获取最新版本和时间*/
fun getMavenVersionAndTime(key: String, mavenroot: String, project: Project): Pair<String, String> {
    val path = key.trim().replace(":", ".", true).split(".").joinToString(separator = "/")
    val runCatching = runCatching {
        val url = "https://maven.aliyun.com/repository/public/$path/maven-metadata.xml"
        project.logger.debug(url)
        val message: String = HttpUtil.createGet(url).setMaxRedirectCount(100).execute().body() ?: ""
        val xmlToMap: MutableMap<String, Any?>? = XmlUtil.xmlToMap(message)
        val version = (xmlToMap?.get("versioning") as? Map<String, Any?>?)?.get("release").toString()
        val lastUpdated = (xmlToMap?.get("versioning") as? Map<String, Any?>?)?.get("lastUpdated").toString()
        version to lastUpdated
    }
    if (runCatching.isSuccess) return runCatching.getOrThrow()
    val file = File(mavenroot, path)
    if (file.exists() && file.isDirectory) {
        val firstOrNull = file.listFiles()?.sortedByDescending { it.name }?.firstOrNull()
        if (firstOrNull != null) {
            return firstOrNull.name to Date(firstOrNull.lastModified()).toString()
        }
    }
    return "null" to "null"
}

inline fun String?.isHikvisionPath(hikvsionMavenNameLikeset: Set<String>): Boolean {
//"com/Hik","crypt","vision/apollo","jef"
//    jch.hikvsionMavenNameLikeset.add("org/postgresql")
//jch.hikvsionMavenNameLikeset.add("com/alibaba")
//jch.hikvsionMavenNameLikeset.add("net/sourceforge")
//jch.hikvsionMavenNameLikeset.add("jxl")
//jch.hikvsionMavenNameLikeset.add("org/easyframe")
//jch.hikvsionMavenNameLikeset.add("commons-codec")
    this ?: return false
    for (like in hikvsionMavenNameLikeset.map { it.trim() }) {
        if (this.startsWith(like, false)) {
            return true
        }
    }
    return false
}